// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


// Interface to Cypress FX2 bus

`define FIFOSIZE 1023 // NB Must be 1 less than 2^N
module fx2_int (input usbclk,
		input reset,
		inout [15:0] usbdata,   // TRISTATE
		input [5:0] usbctl,
		output [5:0] usbrdy,
		output [31:0] txdata,
		input [31:0] rxdata,
		output write_commit,
		output read_done,
		input wire [9:0] txfifolevel,   // 10:0 for 2K lines
		input wire [9:0] rxfifolevel,
		output reg [127:0] control_bus );

   wire WR = usbctl[0];
   wire RD = usbctl[1];
   wire OE = usbctl[2];
   wire WR_CTRL = usbctl[3];

   // These signals allow software to guarantee
   // That there won't be overruns on TX or underruns on RX
   wire have_space = (txfifolevel <= (`FIFOSIZE-128));
   wire have_packet_rdy = (rxfifolevel >= 128);
   
   assign usbrdy[0] = have_space;
   assign usbrdy[1] = have_packet_rdy;
   
   reg [8:0] write_count;
   reg [15:0] tx_holding_reg;
   
   assign txdata = {tx_holding_reg,usbdata};  // FIXME reverse these to swap IQ
   always @(posedge usbclk, posedge reset)
     if(reset)
       tx_holding_reg <= #1 16'd0;
     else if(WR & ~write_count[1])
       tx_holding_reg <= #1 usbdata;
   
   // TX Side bug fix for Cypress 257 bug
   always @(posedge usbclk, posedge reset)
     if(reset)
       write_count <= #1 9'd0;
     else if (~WR)
       write_count <= #1 9'd0;
     else 
       write_count <= #1 write_count + 9'd1;
   
   assign write_commit = WR & ~write_count[8] & write_count[1];

   reg 	read_low;
   always @(posedge usbclk, posedge reset)
     if(reset)
       read_low <= #1 1'b0;
     else if (~RD)
       read_low <= #1 1'b0;
     else 
       read_low <= #1 ~read_low;

   assign read_done = RD & read_low;   // FIXME
   assign usbdata = OE ? (read_low ? rxdata[15:0] : rxdata[31:16]) : 16'bz;
   
   // Write into the control bus
   always @(posedge usbclk, posedge reset)
     if(reset)
       control_bus <= #1 128'b0;
     else if(WR_CTRL)
       case(usbdata[15:8])
	 8'h0: control_bus[7:0] <= usbdata[7:0];
	 8'h1: control_bus[15:8] <= usbdata[7:0];
	 8'h2: control_bus[23:16] <= usbdata[7:0];
	 8'h3: control_bus[31:24] <= usbdata[7:0];
	 8'h4: control_bus[39:32] <= usbdata[7:0];
	 8'h5: control_bus[47:40] <= usbdata[7:0];
	 8'h6: control_bus[55:48] <= usbdata[7:0];
	 8'h7: control_bus[63:56] <= usbdata[7:0];
	 8'h8: control_bus[71:64] <= usbdata[7:0];
	 8'h9: control_bus[79:72] <= usbdata[7:0];
	 8'ha: control_bus[87:80] <= usbdata[7:0];
	 8'hb: control_bus[95:88] <= usbdata[7:0];
	 8'hc: control_bus[103:96] <= usbdata[7:0];
	 8'hd: control_bus[111:104] <= usbdata[7:0];
	 8'he: control_bus[119:112] <= usbdata[7:0];
	 8'hf: control_bus[127:120] <= usbdata[7:0];
       endcase // case(usbdata[15:8])
   
endmodule // fx2_int
