// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

// Top level module for a full setup with DUCs and DDCs

module fullchip
(
 input clk_120mhz,
 input reset,
 input enable_tx,
 input enable_rx,
 input SCLK, 
 input SDI, 
 // output SDO, replaced with clear_status
 input clear_status,
 input SEN, 
 input SLD,
 input wire [11:0] adc1_data,
 input wire [11:0] adc2_data,
 input wire [11:0] adc3_data,
 input wire [11:0] adc4_data,
 output wire [13:0] dac1_data,
 output wire [13:0] dac2_data,
 output wire [13:0] dac3_data,
 output wire [13:0] dac4_data,
 
 output wire adclk0, 
 output wire adclk1,
 
 output [3:0] adc_oeb,
 input [3:0] adc_otr,
 
 output clk_out,
 output [11:0] misc_pins,
 
 // USB interface
 input usbclk,
 input [5:0] usbctl,
 output [5:0] usbrdy,
 inout [15:0] usbdata
 );	

   wire [15:0] usbdata_out;
   wire        adclk;
   wire [7:0]  adc_clock_div, ext_clock_div;
   
   wire [15:0] bb_tx_i0,bb_tx_q0,bb_tx_i1,bb_tx_q1,bb_tx_i2,bb_tx_q2,bb_tx_i3,bb_tx_q3;
   wire [15:0] bb_rx_i0,bb_rx_q0,bb_rx_i1,bb_rx_q1,bb_rx_i2,bb_rx_q2,bb_rx_i3,bb_rx_q3;
   
   wire [31:0] ch1in_freq, ch1out_freq;
   wire [7:0]  interp_rate, decim_rate;
   
   wire [15:0] adc_offset_1,adc_offset_2,adc_offset_3,adc_offset_4;
   wire [13:0] dac_offset_1,dac_offset_2,dac_offset_3,dac_offset_4;

   wire [11:0] tx_debugbus, rx_debugbus;
   wire [31:0] misc_settings;
   
   assign  misc_pins = misc_settings[0] ? tx_debugbus : rx_debugbus;
   
   clk_divider clk_divider_ext(reset,clk_120mhz,clk_out,ext_clock_div);
   clk_divider clk_divider_adc(reset,clk_120mhz,adclk,adc_clock_div);
   
   assign      adclk0 = adclk;
   assign      adclk1 = adclk;
   assign      adc_oeb = 4'h0;
   
   wire        strobe_interp, strobe_decim;
   
   wire [15:0] i_out_0,i_out_1,q_out_0,q_out_1;
   assign      dac1_data = {~i_out_0[15],i_out_0[14:2]} - dac_offset_1;	
   assign      dac2_data = {~q_out_0[15],q_out_0[14:2]} - dac_offset_2;
   assign      dac3_data = {~i_out_1[15],i_out_1[14:2]} - dac_offset_3;
   assign      dac4_data = {~q_out_1[15],q_out_1[14:2]} - dac_offset_4;
   
   wire [15:0] i_in_0 = {{2{adc1_data[11]}},adc1_data,2'b0} - adc_offset_1;
   wire [15:0] q_in_0 = {{2{adc2_data[11]}},adc2_data,2'b0} - adc_offset_2;
   wire [15:0] i_in_1 = {{2{adc3_data[11]}},adc3_data,2'b0} - adc_offset_3;
   wire [15:0] q_in_1 = {{2{adc4_data[11]}},adc4_data,2'b0} - adc_offset_4;
   
   reg 	       enable_tx_reg, enable_rx_reg;
   always @(posedge clk_120mhz or posedge reset)
     if(reset) begin
	enable_tx_reg <= #1 1'b0;
	enable_rx_reg <= #1 1'b0;
     end
     else begin
	enable_tx_reg <= #1 enable_tx;
	enable_rx_reg <= #1 enable_rx;
     end
   
   // Tri-state bus macro
   bustri bustri( .data(usbdata_out),.enabledt(usbctl[2]),.tridata(usbdata) );

   wire [15:0] interp_tx_i0,interp_tx_q0,interp_tx_i1,interp_tx_q1;

   strobe_gen tx_strobe_gen
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_tx_reg),
       .rate(interp_rate),.strobe_in(1'b1),.strobe(strobe_interp) );
   
   tx_buffer tx_buffer
     ( .usbclk(usbclk),.reset(reset),
       .usbdata(usbdata),.WR(usbctl[0]),.have_space(usbrdy[0]),.tx_underrun(usbrdy[2]),
       .tx_i_0(bb_tx_i0),.tx_q_0(bb_tx_q0),
       .tx_i_1(bb_tx_i1),.tx_q_1(bb_tx_q1),
       .tx_i_2(bb_tx_i2),.tx_q_2(bb_tx_q2),
       .tx_i_3(bb_tx_i3),.tx_q_3(bb_tx_q3),
       .txclk(clk_120mhz),.txstrobe(strobe_interp),
       .clear_status(clear_status),		    
       .debugbus(tx_debugbus) );

   cic_interp cic_interp_i_0
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_tx_reg),
       .rate(interp_rate),.strobe_in(strobe_interp),.strobe_out(1'b1),
       .signal_in(bb_tx_i0),.signal_out(interp_tx_i0) );

   cic_interp cic_interp_q_0
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_tx_reg),
       .rate(interp_rate),.strobe_in(strobe_interp),.strobe_out(1'b1),
       .signal_in(bb_tx_q0),.signal_out(interp_tx_q0) );

`ifdef NOCORDIC
   assign i_out_0 = interp_tx_i0;
   assign q_out_0 = interp_tx_q0;
`else
   wire [31:0] phase_t0;
   phase_acc phase_acc_0
     (.clk(clk_120mhz),.reset(reset),.enable(enable_tx_reg),
      .freq(ch1out_freq),.phase(phase_t0) );  
   cordic tx_cordic_0
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_tx_reg), 
       .xi(interp_tx_i0),.yi(interp_tx_q0),.zi(phase_t0[31:16]),
       .xo(i_out_0),.yo(q_out_0),.zo() );
`endif
   
   wire [15:0] decim_tx_i0,decim_tx_q0,decim_tx_i1,decim_tx_q1;

   strobe_gen rx_strobe_gen
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_tx_reg),
       .rate(decim_rate),.strobe_in(adclk),.strobe(strobe_decim) );
   
   rx_buffer rx_buffer
     ( .usbclk(~usbclk),.reset(reset),
       .usbdata(usbdata_out),.RD(usbctl[1]),.have_pkt_rdy(usbrdy[1]),.rx_overrun(usbrdy[3]),
       .rx_i_0(bb_rx_i0),.rx_q_0(bb_rx_q0),
       .rx_i_1(bb_rx_i1),.rx_q_1(bb_rx_q1),
       .rx_i_2(bb_rx_i2),.rx_q_2(bb_rx_q2),
       .rx_i_3(bb_rx_i3),.rx_q_3(bb_rx_q3),
       .rxclk(clk_120mhz),.rxstrobe(strobe_decim),
       .clear_status(clear_status),		    
       .debugbus(rx_debugbus) );

   cic_decim cic_decim_i_0
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_rx_reg),
       .rate(decim_rate),.strobe_in(adclk),.strobe_out(strobe_decim),
       .signal_in(i_in_0),.signal_out(bb_rx_i0) );

   cic_decim cic_decim_q_0
     ( .clock(clk_120mhz),.reset(reset),.enable(enable_rx_reg),
       .rate(decim_rate),.strobe_in(adclk),.strobe_out(strobe_decim),
       .signal_in(q_in_0),.signal_out(bb_rx_q0) );
      
   control_bus control_bus
     ( .serial_clock(SCLK),.serial_data_in(SDI),
       .enable(SEN),.load(SLD),.reset(reset),.serial_data_out(),
       .ch1in_freq(ch1in_freq),.ch2in_freq(),.ch3in_freq(),.ch4in_freq(),
       .ch1out_freq(ch1out_freq),.ch2out_freq(),.ch3out_freq(),.ch4out_freq(),
       .adc_clock_div(adc_clock_div),.ext_clock_div(ext_clock_div),
       .interp_rate(interp_rate),.decim_rate(decim_rate),
       .adc_offset_1(adc_offset_1),.adc_offset_2(adc_offset_2),
       .adc_offset_3(adc_offset_3),.adc_offset_4(adc_offset_4),
       .dac_offset_1(dac_offset_1),.dac_offset_2(dac_offset_2),
       .dac_offset_3(dac_offset_3),.dac_offset_4(dac_offset_4),
       .misc(misc_settings) );
   
endmodule // fullchip
