// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

// New master controller version
// Top level module for a full setup with DUCs and DDCs

module newfc (input clk_120mhz,
	      input reset,
	      input wire [11:0] adc1_data,
	      input wire [11:0] adc2_data,
	      input wire [11:0] adc3_data,
	      input wire [11:0] adc4_data,
	      
	      output wire [13:0] dac1_data,
	      output wire [13:0] dac2_data,
	      output wire [13:0] dac3_data,
	      output wire [13:0] dac4_data,
	      
	      output wire adclk0, 
	      output wire adclk1,
	      
	      output [3:0] adc_oeb,
	      input [3:0] adc_otr,
	      
	      output clk_out,
	      input enable_tx,input enable_rx, // FIXME generate internally
	      output [11:0] misc_pins,
	      
	      // USB interface
	      input usbclk,
	      input [5:0] usbctl,
	      output [5:0] usbrdy,
	      inout [15:0] usbdata,  // BIDIR

	      // RAM Interface
	      // output [17:0] ram_addr,
	      // inout [31:0] ram_data, inout [3:0] ram_parity, // BIDIR
	      // output ram_rw, output ram_we
	      );
	
   wire daclk = clk_120mhz;
   wire adclk;
   wire [7:0] adc_clock_div, ext_clock_div;

   wire [15:0] rx_i_0, rx_q_0;
   wire [15:0] tx_i_0, tx_q_0;
       
   wire [127:0] control_bus;
   
   wire [31:0] ch1in_freq, ch1out_freq;
   wire [3:0]  interp_rate1, decim_rate1, interp_rate2, decim_rate2;

   wire [31:0] tx_fifo_out, rx_fifo_in;
   wire [31:0] tx_fifo_in, rx_fifo_out;

   wire [9:0] txfifolevel_usb, txfifolevel_sys, rxfifolevel_usb, rxfifolevel_sys;
   
   clk_divider clk_divider_ext(reset,clk_120mhz,clk_out,ext_clock_div);
   clk_divider clk_divider_adc(reset,clk_120mhz,adclk,adc_clock_div);
   
   assign adclk0 = adclk;
   assign adclk1 = adclk;
   assign adc_oeb = 4'h0;
	
   wire strobe_interp, strobe_decim;
	
   wire [15:0] i_out;
   wire [15:0] q_out;
   
   assign dac1_data = {~i_out[15],i_out[14:2]};	
   assign dac2_data = {~q_out[15],q_out[14:2]};	
   assign dac3_data = 14'b0;
   assign dac4_data = 14'b0;
   
   reg 	  enable_tx_reg, enable_rx_reg;
   always @(posedge daclk or posedge reset)
     if(reset) enable_tx_reg <= #1 1'b0;
     else enable_tx_reg <= #1 enable_tx;
   
   always @(posedge adclk or posedge reset)
     if(reset) enable_rx_reg <= #1 1'b0;
     else enable_rx_reg <= #1 enable_rx;

   fx2_int fx2_int (.usbclk(usbclk),.reset(reset),
		    .usbdata(usbdata),.usbctl(usbctl),.usbrdy(usbrdy),
		    .txdata(tx_fifo_in),.rxdata(rx_fifo_out),
		    .write_commit(write_commit),.read_done(read_done),
		    .txfifolevel(txfifolevel_usb),.rxfifolevel(rxfifolevel_usb),
		    .control_bus(control_bus) );

   // Only the FIFOs have both usbclk and system clock
   fifo txfifo(.reset(reset),
	       .data(tx_fifo_in),.write(write_commit),.wrclk(usbclk),.wr_used(txfifolevel_usb),// To FX2 Int
	       .q(tx_fifo_out),.read_ack(read_ack),.rdclk(daclk),.rd_used(txfifolevel_sys) // To RAM Int
	       );
      
   fifo rxfifo(.reset(reset),
	       .data(rx_fifo_in),.write(write_req),.wrclk(daclk),.wr_used(rxfifolevel_sys),  // To RAM Int
	       .q(rx_fifo_out),.read_ack(read_done),.rdclk(usbclk),.rd_used(rxfifolevel_usb)   // To FX2 Int
	       );

/*   
   ram_int ram_int(.clock(daclk),.reset(reset),
		   .ram_addr(ram_addr),.ram_rw(ram_rw),.ram_we(ram_we), // RAM
		   .ram_data(ram_data),.ram_parity(ram_parity),  // RAM
		   .tx_i_0(tx_i_0),.tx_q_0(tx_q_0),.tx_i_1(tx_i_1),.tx_q_1(tx_q_1),//DUC
		   .tx_i_2(tx_i_2),.tx_q_2(tx_q_2),.tx_i_3(tx_i_3),.tx_q_3(tx_q_3),//DUC
		   .rx_i_0(rx_i_0),.rx_q_0(rx_q_0),.rx_i_1(rx_i_1),.rx_q_1(rx_q_1),//DDC
		   .rx_i_2(rx_i_2),.rx_q_2(rx_q_2),.rx_i_3(rx_i_3),.rx_q_3(rx_q_3),//DDC
		   .tx_strobe(),.rx_strobe(),  // DUC,DDC
		   .tx_fifo_data(tx_fifo_out),.read_ack(read_ack),.txfifolevel(txfifolevel_sys), //TXFIFO
		   .rx_fifo_data(rx_fifo_in),.write_req(write_req),.rxfifolevel(rxfifolevel_sys), //RXFIFO
		   .num_channels(2'd1),.status() );  // To/from master controller
 */

   assign ch1in_freq = control_bus[31:0];
   assign ch1out_freq = control_bus[63:32];
   assign decim_rate1 = control_bus[67:64];
   assign decim_rate2 = control_bus[71:68];
   assign interp_rate1 = control_bus[75:72];
   assign interp_rate2 = control_bus[79:76];
   
   ddc ddc_ch1(.clock(adclk),.reset(reset),.enable(enable_rx_reg),
	       .strobe(strobe_decim),.freq(ch1in_freq),
	       .rate1(decim_rate1),.rate2(decim_rate2),
	       .i_in({{4{adc1_data[11]}},adc1_data}),.q_in({{4{adc2_data[11]}},adc2_data}),
	       .i_out(rx_i_0),.q_out(rx_q_0) );
   
   duc duc_ch1(.clock(daclk),.reset(reset),.enable(enable_tx_reg),
	       .strobe(strobe_interp),.freq(ch1out_freq),
	       .rate1(interp_rate1),.rate2(interp_rate2),
	       .i_in(tx_i_0),.q_in(tx_q_0),
	       .i_out(i_out),.q_out(q_out) );

   assign rx_fifo_in = {rx_i_0,rx_q_0};
   assign tx_i_0 = tx_fifo_out[31:16];
   assign tx_q_0 = tx_fifo_out[15:0];
   
   assign write_req = strobe_decim;
   assign read_ack = strobe_interp;
   
   
endmodule // newfc
