// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003,2004 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

// Top level module for a full setup with DUCs and DDCs

// Define DEBUG_OWNS_IO_PINS if we're using the daughterboard i/o pins
// for debugging info.  NB, This can kill the m'board and/or d'board if you
// have anything except basic d'boards installed.

// `define DEBUG_OWNS_IO_PINS


module usrp_basic
(
 input master_clk, // CLKOUT2_A
 input RESET,      // NB: Serial Reset ONLY -- also resets the CODECs
 input SCLK,
 input SDI,        // also io_rx_b[15]
 inout SDO,        // NB Careful, should be inout or out!!! also io_rx_a[15]
 input SEN_FPGA,   // also io_tx_a[15]

 input FX2_1,
 output FX2_2,
 output FX2_3,
 
 input wire [11:0] rx_a_a,
 input wire [11:0] rx_b_a,
 input wire [11:0] rx_a_b,
 input wire [11:0] rx_b_b,

 input wire RXSYNC_A,   // FIXME are these inputs?
 input wire RXSYNC_B,   // FIXME do we want to use these?
 
 output wire [13:0] tx_a,
 output wire [13:0] tx_b,

 output wire TXBLANK,  // FIXME do we need this?
 output wire TXSYNC_A,
 output wire TXSYNC_B,
 
  // USB interface
 input usbclk,
 input wire [2:0] usbctl,
 output wire [1:0] usbrdy,
 inout [15:0] usbdata,  // NB Careful, inout

 // These are the general purpose i/o's that go to the daughterboard slots
 inout wire [14:0] io_tx_a,  // bit 15 is SEN_FPGA
 inout wire [14:0] io_tx_b,  // bit 15 is vcxo tuner, but can be used as io
 inout wire [14:0] io_rx_a,  // bit 15 is SDO
 inout wire [14:0] io_rx_b,  // bit 15 is SDI
 input wire vcxo_tune	     // also io_tx_b[15], probably want to be an out
 );	

   wire clk64,clk128;
   
   wire WR = usbctl[0];
   wire RD = usbctl[1];
   wire OE = usbctl[2];

   wire have_space, have_pkt_rdy;
   assign usbrdy[0] = have_space;
   assign usbrdy[1] = have_pkt_rdy;

   wire   tx_underrun, rx_overrun;    
   wire   clear_status = FX2_1;
   assign FX2_2 = rx_overrun;
   assign FX2_3 = tx_underrun;
      
   wire [15:0] usbdata_out;
   
   wire [31:0] ch0rx_freq, ch0tx_freq;
   wire [7:0]  interp_rate, decim_rate;
   
   wire [15:0] tx_debugbus, rx_debugbus;
   wire [31:0] misc_settings;
   
   wire        enable_tx, enable_rx;
   wire        tx_dsp_reset, rx_dsp_reset, tx_bus_reset, rx_bus_reset;
   wire [7:0]  settings;
   
   // Tri-state bus macro
   bustri bustri( .data(usbdata_out),.enabledt(OE),.tridata(usbdata) );

   //   dspclkpll dspclkpll (.inclk0(master_clk),.c0(clk64),.c1(clk128) );
   assign      clk64 = master_clk;

   wire [15:0] ch0tx,ch1tx,ch2tx,ch3tx,ch4tx,ch5tx,ch6tx,ch7tx;
   wire [15:0] ch0rx,ch1rx,ch2rx,ch3rx,ch4rx,ch5rx,ch6rx,ch7rx;
   
   // TX
   wire [15:0] i_out_0,i_out_1,q_out_0,q_out_1;
   wire [15:0] bb_tx_i0,bb_tx_q0,bb_tx_i1,bb_tx_q1,bb_tx_i2,bb_tx_q2,bb_tx_i3,bb_tx_q3;
   
   wire        strobe_interp, tx_sample_strobe;
   wire [7:0]  tx_sample_rate;

   // FIXME implement cool muxing...
   assign      bb_tx_i0 = ch0tx;
   assign      bb_tx_q0 = ch1tx;
   
   strobe_gen da_strobe_gen
     ( .clock(clk64),.reset(tx_dsp_reset),.enable(enable_tx),
       .rate(8'd1),.strobe_in(1'b1),.strobe(tx_sample_strobe) );

   strobe_gen tx_strobe_gen
     ( .clock(clk64),.reset(tx_dsp_reset),.enable(enable_tx),
       .rate(interp_rate),.strobe_in(tx_sample_strobe),.strobe(strobe_interp) );
   
   tx_buffer tx_buffer
     ( .usbclk(usbclk),.bus_reset(tx_bus_reset),.reset(tx_dsp_reset),
       .usbdata(usbdata),.WR(WR),.have_space(have_space),.tx_underrun(tx_underrun),
       .channels(4'd2),
       .tx_i_0(ch0tx),.tx_q_0(ch1tx),
       .tx_i_1(ch2tx),.tx_q_1(ch3tx),
       .tx_i_2(ch4tx),.tx_q_2(ch5tx),
       .tx_i_3(ch6tx),.tx_q_3(ch7tx),
       .txclk(clk64),.txstrobe(strobe_interp),
       .clear_status(clear_status),		    
       .debugbus(tx_debugbus) );

   tx_chain tx_chain_0
     ( .clock(clk64),.reset(tx_dsp_reset),.enable(enable_tx),
       .interp_rate(interp_rate),.sample_strobe(tx_sample_strobe),
       .interpolator_strobe(strobe_interp),.freq(ch0tx_freq),
       .i_in(bb_tx_i0),.q_in(bb_tx_q0),.i_out(i_out_0),.q_out(q_out_0) );

   wire [13:0] tx_a_a = i_out_0[15:2];
   wire [13:0] tx_b_a = q_out_0[15:2];
   wire [13:0] tx_a_b = i_out_1[15:2];
   wire [13:0] tx_b_b = q_out_1[15:2];

   wire txsync = tx_sample_strobe;
   assign TXSYNC_A = txsync;
   assign TXSYNC_B = txsync;
   assign TXBLANK = 1'b0;

   assign tx_a = txsync ? tx_b_a : tx_a_a;
   assign tx_b = txsync ? tx_b_b : tx_a_b;
   
   // RX
   wire        rx_sample_strobe,strobe_decim;
   wire [7:0]  rx_sample_rate;

   // FIXME
   assign      rx_sample_strobe = 1'b1;
   //strobe_gen rx_strobe_gen
   //  ( .clock(clk64),.reset(rx_dsp_reset),.enable(enable_rx),
   //    .rate(rx_sample_rate),.strobe_in(1'b1),.strobe(rx_sample_strobe) );
   
   strobe_gen decim_strobe_gen
     ( .clock(clk64),.reset(rx_dsp_reset),.enable(enable_rx),
       .rate(decim_rate),.strobe_in(rx_sample_strobe),.strobe(strobe_decim) );
   
   wire [15:0] bb_rx_i0,bb_rx_q0,bb_rx_i1,bb_rx_q1,
	       bb_rx_i2,bb_rx_q2,bb_rx_i3,bb_rx_q3;
   
   wire [15:0] adc_offset_0, adc_offset_1, adc_offset_2, adc_offset_3;
   
   // Buffer at input to chip, then subtract dc offset
   reg [15:0]  i_in_0_buf, q_in_0_buf, i_in_1_buf, q_in_1_buf;
   reg [15:0]  i_in_0, q_in_0, i_in_1, q_in_1;
   always @(posedge clk64)
     begin
	i_in_0_buf <= #1 {{2{rx_a_a[11]}},rx_a_a,2'b0};
	q_in_0_buf <= #1 {{2{rx_b_a[11]}},rx_b_a,2'b0};
	i_in_1_buf <= #1 {{2{rx_a_b[11]}},rx_a_b,2'b0};
	q_in_1_buf <= #1 {{2{rx_b_b[11]}},rx_b_b,2'b0};

	i_in_0 <= #1 i_in_0_buf - adc_offset_0;
	q_in_0 <= #1 q_in_0_buf - adc_offset_1;
	i_in_1 <= #1 i_in_1_buf - adc_offset_2;
	q_in_1 <= #1 q_in_1_buf - adc_offset_3;
     end

   wire loopback = settings[0];
   wire counter = settings[1];

   reg [15:0] debug_counter;
   
   reg [15:0] loopback_i_0,loopback_q_0;

   always @(posedge clk64)
     if(rx_dsp_reset)
       debug_counter <= #1 16'd0;
     else if(~enable_rx)
       debug_counter <= #1 16'd0;
     else if(strobe_decim)
       debug_counter <=#1 debug_counter + 16'd2;
   
   always @(posedge clk64)
     if(strobe_interp)
       begin
	  loopback_i_0 <= #1 ch0tx;
	  loopback_q_0 <= #1 ch1tx;
       end
   
   assign ch0rx = counter ? debug_counter : loopback ? loopback_i_0 : bb_rx_i0;
   assign ch1rx = counter ? debug_counter + 16'd1 : loopback ? loopback_q_0 : bb_rx_q0;
   assign ch2rx = 16'd0;
   assign ch3rx = 16'd0;
   assign ch4rx = 16'd0;
   assign ch5rx = 16'd0;
   assign ch6rx = 16'd0;
   assign ch7rx = 16'd0;
   
   rx_buffer rx_buffer
     ( .usbclk(usbclk),.bus_reset(rx_bus_reset),.reset(rx_dsp_reset),
       .usbdata(usbdata_out),.RD(RD),.have_pkt_rdy(have_pkt_rdy),.rx_overrun(rx_overrun),
       .channels(4'd2),
       .ch_0(ch0rx),.ch_1(ch1rx),
       .ch_2(ch2rx),.ch_3(ch3rx),
       .ch_4(ch4rx),.ch_5(ch5rx),
       .ch_6(ch6rx),.ch_7(ch7rx),
       .rxclk(clk64),.rxstrobe(strobe_decim),
       .clear_status(clear_status),		    
       .debugbus(rx_debugbus) );

   rx_chain rx_chain_0
     ( .clock(clk64),.reset(rx_dsp_reset),.enable(enable_rx),
       .decim_rate(decim_rate),.sample_strobe(rx_sample_strobe),
       .decimator_strobe(strobe_decim),.freq(ch0rx_freq),
       .i_in(i_in_0),.q_in(q_in_0),.i_out(bb_rx_i0),.q_out(bb_rx_q0));
   
   // Control, clock and reset
   wire [7:0]  master_controls;
   wire [15:0] io_0, io_1, io_2, io_3;
   
   serial_io serial_io
     ( .serial_clock(SCLK),.serial_data_in(SDI),
       .enable(SEN_FPGA),.reset(~RESET),.serial_data_out(SDO),
       .ch0rx_freq(ch0rx_freq),.ch1rx_freq(),.ch2rx_freq(),.ch3rx_freq(),
       .ch0tx_freq(ch0tx_freq),.ch1tx_freq(),.ch2tx_freq(),.ch3tx_freq(),
       .interp_rate(interp_rate),.decim_rate(decim_rate),
       .tx_sample_rate(tx_sample_rate),.rx_sample_rate(rx_sample_rate),
       .adc_offset_0(adc_offset_0),.adc_offset_1(adc_offset_1),
       .adc_offset_2(adc_offset_2),.adc_offset_3(adc_offset_3),
       .master_controls(master_controls),
       .io_0(io_0),.io_1(io_1),.io_2(io_2),.io_3(io_3),
       .settings(settings) );

   master_control master_control
//     ( .master_clk(clk128),.usbclk(usbclk),
     ( .master_clk(clk64),.usbclk(usbclk),
       .master_controls(master_controls),
       .tx_bus_reset(tx_bus_reset),.rx_bus_reset(rx_bus_reset),
       .tx_dsp_reset(tx_dsp_reset),.rx_dsp_reset(rx_dsp_reset),
       .enable_tx(enable_tx),.enable_rx(enable_rx) );
   
   // DEBUG stuff

`ifdef DEBUG_OWNS_IO_PINS
   wire        dummy_0, dummy_1, dummy_2, dummy_3;

   bidir_reg db_bidir_0
     ( .tristate ({dummy_0, io_tx_a}), .oe(16'hffff),
       .reg_val({1'b0,OE,usbdata_out[13:0]}));
   
   bidir_reg db_bidir_1
     ( .tristate ({dummy_1, io_rx_a}), .oe(16'hffff),
       .reg_val({1'b0,rx_debugbus[14:0]}));
   
   bidir_reg db_bidir_2
     ( .tristate ({dummy_2, io_tx_b}), .oe(16'hffff),
       .reg_val({1'b0,bb_rx_i0[15:1]}));
   
   bidir_reg db_bidir_3
     ( .tristate ({dummy_3, io_rx_b}), .oe(16'hffff),
       .reg_val({1'b0,bb_rx_q0[15:1]}));

`else
   assign      io_tx_a = io_0[14:0];
   assign      io_rx_a = io_1[14:0];
   assign      io_tx_b = io_2[14:0];
   assign      io_rx_b = io_3[14:0];
`endif
   

   
endmodule // usrp_basic
